#include "board.h"

#define ROWS ( 4 )
#define COLUMNS ( 4 )
#define TILES (ROWS * COLUMNS)

#define BLANK_TILE_NUMBER ( 15 )

#define TILE_SIZE ( 64.0f )

//
// Horizontal screen layout:
// 
//      [padding] [guide] [padding] [playarea] [padding]
//
#define HORIZONTAL_PADDING ( ( SCREEN_WIDTH - 2.0f * COLUMNS * TILE_SIZE ) / 3.0f )

#define GUIDE_TOP ( ( SCREEN_HEIGHT - ROWS * TILE_SIZE ) / 2.0f )
#define GUIDE_LEFT ( HORIZONTAL_PADDING )

#define PLAYAREA_TOP ( GUIDE_TOP )
#define PLAYAREA_LEFT ( GUIDE_LEFT + COLUMNS * TILE_SIZE + HORIZONTAL_PADDING )

// holds tile numbers, between 0 and 15
int p_tiles[ROWS][COLUMNS];

f32 p_getTileXFromCol( int col ) {
	return PLAYAREA_LEFT + TILE_SIZE * (float)col;
}

f32 p_getTileYFromRow( int row ) {
	return PLAYAREA_TOP + TILE_SIZE * (float)row;
}

int p_getColFromTileNum( int tileNum ) {
	return tileNum % COLUMNS;
}

int p_getRowFromTileNum( int tileNum ) {
	return tileNum / COLUMNS;
}

void p_shuffle() {
	int i;
	
	// starting with the board in a solution configuration,
	// randomly perform moves on it many times
	// this ensures that the board will be solvable, the player merely
	// needing to work backwards from where the shuffle moves left off
	for( i = 0; i < 2000; i++ ) {
		board_move( rand() % 4 );
	}
}

void board_init() {
	int row, col;
	
	for( row = 0; row < ROWS; row++ ) {
		for( col = 0; col < COLUMNS; col++ ) {
			p_tiles[row][col] = row * COLUMNS + col;
		}
	}
	
	p_shuffle();
}

void board_render() {
	int row, col;
	int rowInOriginalPicture, colInOriginalPicture;
	f32 textureSectionWidth = 1.0f / COLUMNS;
	f32 textureSectionHeight = 1.0f / ROWS;
	f32 textureLeft, textureTop;
	
	// tiles
	for( row = 0; row < ROWS; row++ ) {
		for( col = 0; col < COLUMNS; col++ ) {
			rowInOriginalPicture = p_getRowFromTileNum( p_tiles[row][col] );
			colInOriginalPicture = p_getColFromTileNum( p_tiles[row][col] );
			
			textureLeft = (float)colInOriginalPicture * textureSectionWidth;
			textureTop = (float)rowInOriginalPicture * textureSectionHeight;
			
			display_draw( 
				p_getTileXFromCol( col ), 
				p_getTileYFromRow( row ), 
				TILE_SIZE, 
				TILE_SIZE,
				textureLeft,
				textureTop,
				textureLeft + textureSectionWidth,
				textureTop + textureSectionHeight
				);
		}
	}
	
	// guide
	display_draw_full( 
		GUIDE_LEFT, 
		GUIDE_TOP, 
		TILE_SIZE * ROWS, 
		TILE_SIZE * COLUMNS 
		);
}

void p_findBlankTile( int* r, int* c ) {
	int row, col;
	
	for( row = 0; row < ROWS; row++ ) {
		for( col = 0; col < COLUMNS; col++ ) {
			if( BLANK_TILE_NUMBER == p_tiles[row][col] ) {
				*r = row;
				*c = col;
				return;
			}
		}
	}
}

void p_moveUp() {
	int row = 0, col = 0;
	p_findBlankTile( &row, &col );
	if( ROWS - 1 == row ) {
		return;
	}
	p_tiles[row][col] = p_tiles[row+1][col];
	p_tiles[row+1][col] = BLANK_TILE_NUMBER;
}

void p_moveDown() {
	int row = 0, col = 0;
	p_findBlankTile( &row, &col );
	if( 0 == row ) {
		return;
	}
	p_tiles[row][col] = p_tiles[row-1][col];
	p_tiles[row-1][col] = BLANK_TILE_NUMBER;
}

void p_moveLeft() {
	int row = 0, col = 0;
	p_findBlankTile( &row, &col );
	if( COLUMNS - 1 == col ) {
		return;
	}
	p_tiles[row][col] = p_tiles[row][col+1];
	p_tiles[row][col+1] = BLANK_TILE_NUMBER;
}

void p_moveRight() {
	int row = 0, col = 0;
	p_findBlankTile( &row, &col );
	if( 0 == col ) {
		return;
	}
	p_tiles[row][col] = p_tiles[row][col-1];
	p_tiles[row][col-1] = BLANK_TILE_NUMBER;
}

void board_move( Move move ) {
	switch( move ) {
		case Left:
			p_moveLeft();
			break;
		case Right:
			p_moveRight();
			break;
		case Up:
			p_moveUp();
			break;
		case Down:
			p_moveDown();
			break;
	}
}

int board_hasWon() {
	int last = -99;
	int row, col;
	
	for( row = 0; row < ROWS; row++ ) {
		for( col = 0; col < COLUMNS; col++ ) {
			if( last > p_tiles[row][col] ) {
				return 0;
			}
			last = p_tiles[row][col];
		}
	}
	
	return 1;
}
